@@ -19,6 +19,8 @@ import re
 import time
 import urlparse
 import subprocess
+import gettext
+import socket
 
 import dbus
 
@@ -26,6 +28,7 @@ import fmms_config as fMMSconf
 import dbhandler as DBHandler
 from mms.message import MMSMessage
 from mms import mms_pdu
+from wappushhandler import MMSSender
 
 #TODO: constants.py?
 MSG_DIRECTION_IN = 0
@@ -33,6 +36,10 @@ MSG_DIRECTION_OUT = 1
 MSG_UNREAD = 0
 MSG_READ = 1
 
+_ = gettext.gettext
+gettext.bindtextdomain('fmms','/opt/fmms/share/locale/')
+gettext.textdomain('fmms')
+
 class fMMS_controller():
 	
 	def __init__(self):
@@ -42,6 +49,7 @@ class fMMS_controller():
 		self._pushdir = self.config.get_pushdir()
 		self._outdir = self.config.get_outdir()
 		self.store = DBHandler.DatabaseHandler(self)
+		self.ui = False
 	
 	def clean_url(self, url):
 		m = re.search(r"http\:\/\/(?i)", url)
@@ -97,15 +105,43 @@ class fMMS_controller():
 			mtime = intime
 		return mtime
 	
+	def send_mms(self, to, subject, message, attachment, sender):
+		try:
+			sender = MMSSender(to, subject, message, attachment, sender, setupConn=True, controller=self)
+		except:
+			msg = "%s" % (gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed"))
+			return (-1, msg)
+		try:
+			(status, reason, output, parsed) = sender.sendMMS()
+
+			if parsed == True and "Response-Status" in output:
+				if output['Response-Status'] == "Ok":
+					log.info("message seems to have sent AOK!")
+					return (0, "OK")
+
+			errstr = gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed")
+			message = str(status) + "_" + str(reason)
+			reply = str(output)
+			msg = "%s\nMMSC: %s\nBODY: %s" % (errstr, message, reply)
+			return (-1, msg)
+		except socket.error, exc:
+			log.exception("sender failed due to connection error")
+			errstr = gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed")
+			errhelp = _("Please check your settings.")
+			msg = "%s\n%s\n%s" % (errstr, exc, errhelp)
+			return (-1, msg)
+			#raise
+		except Exception, exc:
+			log.exception("Sender failed.")
+			msg = "%s\n%s" % (gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed"), exc)
+			return (-1, msg)
+			#raise
+	
 	def decode_mms_from_push(self, binarydata):
 		""" decodes the given mms """
 		decoder = mms_pdu.MMSDecoder()
 		wsplist = decoder.decodeCustom(binarydata)
-
 		sndr, url, trans_id = None, None, None
-		bus = dbus.SystemBus()
-		proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
-		interface = dbus.Interface(proxy, dbus_interface='org.freedesktop.Notifications')
 
 		try:
 			url = wsplist["Content-Location"]
@@ -115,7 +151,10 @@ class fMMS_controller():
 			log.info("transid: %s", trans_id)
 		except Exception, e:
 			log.exception("no content-location/transid in push; aborting: %s %s", type(e), e)
-			interface.SystemNoteInfoprint("Failed to parse SMS PUSH.")
+			bus = dbus.SystemBus()
+			proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
+			interface = dbus.Interface(proxy, dbus_interface='org.freedesktop.Notifications')
+			interface.SystemNoteInfoprint(gettext.ldgettext('modest', "mail_ni_ui_folder_get_msg_folder_error"))
 			raise
 		try:
 			sndr = wsplist["From"]
@@ -185,7 +224,7 @@ class fMMS_controller():
 		""" decodes and saves the binary mms"""
 		# Decode the specified file
 		# This also creates all the parts as files in path
-		log.info("decode_binary_mms running")
+		log.info("decode_binary_mms running: %s", str(path))
 		try:
 			message = MMSMessage.fromFile(path + "/message")
 		except Exception, e:
@@ -242,8 +281,8 @@ class fMMS_controller():
 		
 		from wappushhandler import PushHandler
 		p = PushHandler()
-		path = p._get_mms_message(url, trans_id)
-		log.info("decoding mms... %s", trans_id)
+		path = p._get_mms_message(url, trans_id, self)
+		log.info("path: %s", path)
 		message = self.decode_binary_mms(path)
 		log.info("storing mms...%s", trans_id)
 		mmsid = self.store_mms_message(pushid, message)
@@ -300,11 +339,23 @@ class fMMS_controller():
 		self.delete_mms_message(transactionid)
 		self.delete_push_message(transactionid)
 	
+	def save_draft(self, rcpt, text, attachment):
+		if not rcpt:
+			rcpt = ""
+		if not text:
+			text = ""
+		if not attachment:
+			attachment = ""
+		self.store.save_draft(rcpt, text, attachment)
+
+	def get_draft(self):
+		return self.store.get_draft()
+
 	def validate_phonenumber_email(self, nr):
 		nr = str(nr)
 		nr = nr.replace("+", "")
 		nr = nr.replace(" ", "")
-		if re.search(r"(\D)+", nr) == None or "@" in nr:
+		if re.search(r"(\D)+", nr) == None or "@" in nr or ";" in nr:
 			return True
 		else:
 		 	return False
@@ -409,7 +460,7 @@ class fMMS_controller():
 		mcc = str(int(mcc))
 		mnc = str(int(mnc))
 		if mcc == "240" and mnc == "7":
-			if operatorname == "Tele2" or operatorname == "Comviq":
+			if "Tele2" in operatorname:
 				settings = {}
 				settings['apn'] = "internet.tele2.se"
 				settings['user'] = ""
@@ -423,11 +474,16 @@ class fMMS_controller():
 				return settings
 		return False
 
+	def reset_all_settings(self):
+		self.config.reset_all_settings()
+
+
 class Locker:
 	def __init__(self, fn):
 		self.fn = fn
 		self.fd = None
 		self.pid = os.getpid()
+		self.failcounter = 0
 	
 	def lock(self):
 		try:
@@ -437,6 +493,16 @@ class Locker:
 		except OSError:
 			# we failed to lock
 			self.fd = None
+			self.failcounter += 1
+			# after 5 unsuccessful locks we check the owner
+			# is still alive
+			if self.failcounter > 4:
+				pid = open(self.fn, 'r').read()
+				try:
+					os.kill(int(pid), 0)
+				except OSError, e:
+					# no such process, remove the lock
+					os.remove(self.fn)
 			return 0
 	
 	def unlock(self):
